function [x, u, cost] = gtvd(y, rho1, rho2, Nit)
% [x, u, cost] = gtvd(y, rho1, rho2, Nit)
%
% Generalized Total Variation Denoising
%
% The method is based on pairwise group sparse representation of
% second derivative (using synthesis group sparsity).
%
% INPUT
%   y - noisy signal
%   rho1, rho2 -  knot regularization parameters (positive)
%   Nit - number of iterations
%
% OUTPUT
%   x - denoised signal
%   u - sparse representation
%   cost - cost function history

% Reference:
% 'Generalized Total Variation: Tying the Knots'
% Ivan Selesnick, IEEE Signal Processing Letters, 22(11):2009-2013, November 2015. 
% Email: selesi@nyu.edu

y = y(:);                                   % Convert to column vector
cost = zeros(1, Nit);                       % Cost function history
N = length(y);

e = ones(N, 1);
D = spdiags([e -2*e e], [0 1 2], N-2, N);   % second order difference
DDT = D * D';

S = kron(speye(N-2), [1 1]);
S = S(:, 2:end-1);

P = [rho1 rho1; rho2 -rho2];
L = N - 3;        % Number of groups = L = N-3

A = S * kron(speye(L), inv(P) );
Dy = D*y;
w = ones(L,1);      % Initialization

for k = 1:Nit
    G = spdiags(kron(w, [1; 1]), 0, 2*L, 2*L);
    mu = (DDT + A * G * A') \ Dy;
    
    x = y - D' * mu;
    u = G *  A' * mu;
    w = sqrt(u(1:2:end).^2 + u(2:2:end).^2);
    
    %   max(abs(A*u - D*x)) % verify (should be zero)
    cost(k) = 0.5 * sum(abs( x - y ).^2) + sum( w );
    
end

